@@ -1,3 +1,4 @@
+from numpy.random.mtrand import normal, sample
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
@@ -8,8 +9,166 @@ import pickle
 import numpy as numpy
 import os
 from ingradient_library.patch_transform import Batch_Affine_3D
+from ingradient_library.preprocessing import *
+from ingradient_library.transform import *
+from torchvision.transforms import RandomCrop
+import h5py
 
 
+def random_axis_sample(images, seg, axis, mode = 'over'):
+    if isinstance(images, np.ndarray) or isinstance(seg, np.ndarray):
+        images = torch.tensor(images)
+        seg = torch.tensor(seg)
+    if mode == 'over':
+        candidates = torch.where(seg != 0)[axis]
+        index = torch.randint(low = candidates.min(), high = candidates.max() + 1, size = (1,)).item()
+    
+    elif mode == 'random':
+        index = torch.randint(low = 0, high = seg.shape[axis], size = (1,)).item()
+
+    if axis == 0:
+        images = images[:,index]
+        seg = seg[index]
+    
+    elif axis == 1:
+        images = images[:, :, index]
+        seg = seg[:, index]
+    
+    elif axis == 2:
+        images = images[:, :, :, index]
+        seg = seg[:, :, index]
+
+    return images, seg
+
+def set_direction(img, direction):
+    if isinstance(img, np.ndarray):
+        img = torch.tensor(img)
+    img = img.permute(2,1,0)
+    img = img.numpy()
+    direction = np.array(direction).reshape(3,3)
+    inverse = np.where(direction == -1)[0]
+    permute = np.where(direction != 0)[1]
+    img = img.transpose(permute)
+    if 0 in inverse:
+        img = img[::-1, :, :]
+
+    if 1 in inverse:
+        img = img[:, ::-1, :]
+
+    if 2 in inverse:
+        img = img[:, :, ::-1]
+
+    return torch.tensor(img.copy())
+
+
+class Dataset2D(Dataset):
+    def __init__(self, root_dir, axis, mode, hdf_img_keys = None, train = True, hdf_seg_key = None ,sampler = None, size = (520, 520),
+                oversampling = True, device = None, normalizer = None, transform = None, resample = None, correct_direction = True):
+
+        self.file_name = []
+        self.info_name = []
+        for file_name in os.listdir(root_dir):
+            if mode == 'npz':
+                if 'npz' in file_name and not 'py' in file_name:
+                    self.file_name.append(file_name)
+            
+                elif 'pkl' in file_name and not 'py' in file_name:
+                    self.info_name.append(file_name)
+            elif mode == 'hdf':
+                if not 'py' in file_name and 'hdf' in file_name:
+                    self.file_name.append(file_name)
+        
+        self.root_dir = root_dir
+        self.axis = axis
+        self.mode = mode
+        self.hdf_img_keys = hdf_img_keys
+        self.hdf_seg_key = hdf_seg_key
+        self.sampler = sampler
+        self.oversampling = oversampling
+        self.device = device
+        self.normalizer = normalizer
+        self.transform = transform
+        self.resample = resample
+        self.correct_direction = correct_direction
+        self.train = train
+        self.crop = RandomCrop(size = size, pad_if_needed = True)
+        
+    def __len__(self):
+        return len(self.file_name)
+    
+    def resample(self, x, info_data, y = None):
+        if self.train:
+                if isinstance(y, np.ndarray):
+                    y = torch.tensor(y.copy())
+                result = self.resample(torch.vstack((x, y.unsqueeze(0))), info_data)
+                x = result[:-1]
+                y = result[-1].squeeze(0)
+                return x, y
+            
+        else:
+            x = self.resample(x, info_data)
+            return x, y
+    
+    def __getitem__(self, idx):
+        current_file = os.path.join(self.root_dir, self.file_name[idx])
+        
+        if self.mode == 'npz':
+            info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
+            info_data = pickle.load(info_file)
+            info_file.close()
+            data = np.load(os.path.join(self.root_dir, self.file_name[idx]))
+            x = data['x']
+            y = data['y']
+
+        elif self.mode == 'hdf':
+            hdf_file = h5py.File(current_file , 'r')
+            first_check = True
+            for h_key in self.hdf_img_keys:
+                if first_check:
+                    x = np.array(hdf_file[h_key])
+                    first_check = False
+                else:
+                    x = np.stack((np.expand_dims(x, axis = 0), np.expand_dims(hdf_file[h_key], axis = 0)))
+            y = np.array(hdf_file[self.hdf_seg_key[0]])
+        
+
+        if self.correct_direction:
+            for i in range(len(x)):
+                if i == 0:
+                    result = set_direction(x[i], info_data['direction']).unsqueeze(0)
+                else:
+                    result = torch.vstack((result, set_direction(x[i], info_data['direction']).unsqueeze(0)))
+            x = result 
+            if self.train:
+                y = set_direction(y, info_data['direction'])
+
+        if self.normalizer != None:
+            x = self.normalizer(x)
+        
+        if self.resample != None:
+            x, y = self.resample(x, info_data, y)
+        
+        if self.sampler != None:
+            x, y = self.sampler(x, y)
+        
+
+        if self.axis == None:
+            return x, y
+        if self.oversampling:
+            temp = np.random.uniform(0, 1)
+            if temp < 0.5:
+                x, y = random_axis_sample(x, y, self.axis, mode = 'over')
+            else:
+                x, y = random_axis_sample(x, y, self.axis, mode = 'random')
+        
+        else:
+            x, y = random_axis_sample(x, y, self.axis, mode = 'random')
+
+        x, y = self.crop(image = x, target = y)
+        return x, y
+
+        
+
 class CustomDataset(Dataset):
     def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = True, resample = None):
         self.root_dir = root_dir
