@@ -4,8 +4,38 @@ import pickle
 import os
 import SimpleITK as sitk
 import torch.nn.functional as F
+import cc3d
 import copy
 
+
+
+class CT_Lung_Cropping(object):
+    def __init__(self, clip = False, threshold_clip = [-900, -200 ], threshold_cut = [-1000, -350]):
+        self.clip = clip
+        self.threshold_clip = threshold_clip
+        self.threshold_cut = threshold_cut
+    
+    def __call__(self, image):
+        if self.clip:
+            clip_img = np.clip(image, self.threshold_clip[0], self.threshold_clip[1])
+        else:
+            clip_img = image
+        mask =  (clip_img > self.threshold_cut[0]) * (clip_img < self.threshold_cut[1])
+        if not isinstance(mask, np.ndarray):
+            mask = mask.numpy()
+        conected_component_mask = cc3d.connected_components(mask, connectivity = 26)
+        count = np.unique(conected_component_mask, return_counts = True)[1]
+        lung_index = np.argsort(count)[::-1]
+        if len(lung_index) > 2:
+            lung_index = lung_index[2]
+        
+        else:
+            lung_index = lung_index[-1]
+        lung_max = np.max(np.where(conected_component_mask == lung_index), axis = 1)
+        lung_min = np.min(np.where(conected_component_mask == lung_index), axis = 1)
+        
+        return lung_min, lung_max
+
 class Normalizer(object):
     def __init__(self, percentile_clipping_ratio = None):
         self.percentile = percentile_clipping_ratio
@@ -140,8 +170,8 @@ class Get_target_spacing(object):
         self.spaces = None
         self.first = True
         self.anisotropy_threshold = anisotropy_threshold
-        self.isotropy_percentile_value = 0.50
-        self.anisotropy_percentile_value = 0.90
+        self.isotropy_percentile_value = 50
+        self.anisotropy_percentile_value = 90
         self.image_dimension = image_dimension
 
     def run(self, dir_path, dtype = 'pkl'):
@@ -200,3 +230,46 @@ class Get_target_spacing(object):
         return np.where((np.max(self.spaces, axis = 0) / np.min(self.spaces, axis = 0)) > self.anisotropy_threshold)[0].astype(int)
     
 
+
+class ResamplingV2(object):
+    def __init__(self, anisotropy_axis_index = [], target_spacing = [1.0, 1.0, 1.0], random = False, device = None):
+        self.device = device
+        self.anisotropy_axis_index = self.new_anisotropy_axis(anisotropy_axis_index)
+        self.target_spacing = np.array(target_spacing)
+        self.random = random
+    
+    def new_anisotropy_axis(self, array):
+        for i in range(len(array)):
+            if array[i] == 2:
+                array[i] = 0
+            else:
+                array[i] += 1
+        return array
+
+
+    def __call__(self, img, info_data, seg = None):
+        if isinstance(img, np.ndarray):
+            img = torch.from_numpy(img.copy())
+        if img.device.index != self.device and self.device:
+            img = img.to(self.device)
+        
+        if isinstance(seg, np.ndarray):
+            seg = torch.from_numpy(seg.copy())
+        if img.device.index != self.device and self.device:
+            seg = img.to(self.device)
+        
+        if isinstance(info_data, dict):
+            original_spacing = info_data['spacing']
+        else:
+            original_spacing = info_data
+        
+        if self.random:
+            self.target_spacing = original_spacing * np.random.uniform(0.20, 5.0, size = np.array(original_spacing).shape)
+        new_shape = list((original_spacing/self.target_spacing * np.array([img.shape[-3], img.shape[-2], img.shape[-1]])).astype(int))
+        
+        img = F.interpolate(img.unsqueeze(0), new_shape, mode = 'trilinear').squeeze(0)
+        
+        if seg != None:
+            seg = F.interpolate(seg.unsqueeze(0).unsqueeze(0), new_shape, mode = 'nearest').squeeze(0).squeeze(0)
+
+        return img, seg
